<?php
/* Copyright (C) 2023 SuperAdmin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * \file    core/triggers/interface_99_modConditionnement_ConditionnementTriggers.class.php
 * \ingroup conditionnement
 * \brief   Example trigger.
 *
 * Put detailed description here.
 *
 * \remarks You can create other triggers by copying this one.
 * - File name should be either:
 *      - interface_99_modConditionnement_MyTrigger.class.php
 *      - interface_99_all_MyTrigger.class.php
 * - The file must stay in core/triggers
 * - The class name must be InterfaceMytrigger
 */

require_once DOL_DOCUMENT_ROOT.'/core/triggers/dolibarrtriggers.class.php';
require_once DOL_DOCUMENT_ROOT.'/product/stock/class/mouvementstock.class.php';
require_once DOL_DOCUMENT_ROOT.'/expedition/class/expedition.class.php';
require_once DOL_DOCUMENT_ROOT.'/custom/conditionnement/class/carton.class.php';

/**
 *  Class of triggers for Conditionnement module
 */
class InterfaceConditionnementTriggers extends DolibarrTriggers
{
	/**
	 * Constructor
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;

		$this->name = preg_replace('/^Interface/i', '', get_class($this));
		$this->family = "demo";
		$this->description = "Conditionnement triggers.";
		// 'development', 'experimental', 'dolibarr' or version
		$this->version = 'development';
		$this->picto = 'conditionnement@conditionnement';
	}

	/**
	 * Trigger name
	 *
	 * @return string Name of trigger file
	 */
	public function getName()
	{
		return $this->name;
	}

	/**
	 * Trigger description
	 *
	 * @return string Description of trigger file
	 */
	public function getDesc()
	{
		return $this->description;
	}


	/**
	 * Function called when a Dolibarrr business event is done.
	 * All functions "runTrigger" are triggered if file
	 * is inside directory core/triggers
	 *
	 * @param string 		$action 	Event action code
	 * @param CommonObject 	$object 	Object
	 * @param User 			$user 		Object user
	 * @param Translate 	$langs 		Object langs
	 * @param Conf 			$conf 		Object conf
	 * @return int              		<0 if KO, 0 if no triggered ran, >0 if OK
	 */
	public function runTrigger($action, $object, User $user, Translate $langs, Conf $conf)
	{
		if (empty($conf->conditionnement) || empty($conf->conditionnement->enabled)) {
			return 0; // If module is not enabled, we do nothing
		}

		// Put here code you want to execute when a Dolibarr business events occurs.
		// Data and type of action are stored into $object and $action

		// You can isolate code for each action in a separate method: this method should be named like the trigger in camelCase.
		// For example : COMPANY_CREATE => public function companyCreate($action, $object, User $user, Translate $langs, Conf $conf)
		$methodName = lcfirst(str_replace(' ', '', ucwords(str_replace('_', ' ', strtolower($action)))));
		$callback = array($this, $methodName);
		if (is_callable($callback)) {
			dol_syslog(
				"Trigger '".$this->name."' for action '$action' launched by ".__FILE__.". id=".$object->id
			);

			return call_user_func($callback, $action, $object, $user, $langs, $conf);
		};
		// STOCK_MOVEMENT, SHIPPING_VALIDATE,
		/*echo '<script type="text/javascript">window.alert("'.$action.'");</script>';*/
		// Or you can execute some code here
		switch ($action) {
			// Users
			//case 'USER_CREATE':
			//case 'USER_MODIFY':
			//case 'USER_NEW_PASSWORD':
			//case 'USER_ENABLEDISABLE':
			//case 'USER_DELETE':

			// Actions
			//case 'ACTION_MODIFY':
			//case 'ACTION_CREATE':
			//case 'ACTION_DELETE':

			// Groups
			//case 'USERGROUP_CREATE':
			//case 'USERGROUP_MODIFY':
			//case 'USERGROUP_DELETE':

			// Companies
			//case 'COMPANY_CREATE':
			//case 'COMPANY_MODIFY':
			//case 'COMPANY_DELETE':

			// Contacts
			//case 'CONTACT_CREATE':
			//case 'CONTACT_MODIFY':
			//case 'CONTACT_DELETE':
			//case 'CONTACT_ENABLEDISABLE':

			// Products
			//case 'PRODUCT_CREATE':
			//case 'PRODUCT_MODIFY':
			//case 'PRODUCT_DELETE':
			//case 'PRODUCT_PRICE_MODIFY':
			//case 'PRODUCT_SET_MULTILANGS':
			//case 'PRODUCT_DEL_MULTILANGS':

			//Stock mouvement
			//case 'STOCK_MOVEMENT':

			//MYECMDIR
			//case 'MYECMDIR_CREATE':
			//case 'MYECMDIR_MODIFY':
			//case 'MYECMDIR_DELETE':

			// Customer orders
			//case 'ORDER_CREATE':
			//case 'ORDER_MODIFY':
			//case 'ORDER_VALIDATE':
			//case 'ORDER_DELETE':
			//case 'ORDER_CANCEL':
			//case 'ORDER_SENTBYMAIL':
			//case 'ORDER_CLASSIFY_BILLED':
			//case 'ORDER_SETDRAFT':
			//case 'LINEORDER_INSERT':
			//case 'LINEORDER_UPDATE':
			//case 'LINEORDER_DELETE':

			// Supplier orders
			//case 'ORDER_SUPPLIER_CREATE':
			//case 'ORDER_SUPPLIER_MODIFY':
			//case 'ORDER_SUPPLIER_VALIDATE':
			//case 'ORDER_SUPPLIER_DELETE':
			//case 'ORDER_SUPPLIER_APPROVE':
			//case 'ORDER_SUPPLIER_REFUSE':
			//case 'ORDER_SUPPLIER_CANCEL':
			//case 'ORDER_SUPPLIER_SENTBYMAIL':
			//case 'ORDER_SUPPLIER_DISPATCH':
			//case 'LINEORDER_SUPPLIER_DISPATCH':
			//case 'LINEORDER_SUPPLIER_CREATE':
			//case 'LINEORDER_SUPPLIER_UPDATE':
			//case 'LINEORDER_SUPPLIER_DELETE':

			// Proposals
			//case 'PROPAL_CREATE':
			//case 'PROPAL_MODIFY':
			//case 'PROPAL_VALIDATE':
			//case 'PROPAL_SENTBYMAIL':
			//case 'PROPAL_CLOSE_SIGNED':
			//case 'PROPAL_CLOSE_REFUSED':
			//case 'PROPAL_DELETE':
			//case 'LINEPROPAL_INSERT':
			//case 'LINEPROPAL_UPDATE':
			//case 'LINEPROPAL_DELETE':

			// SupplierProposal
			//case 'SUPPLIER_PROPOSAL_CREATE':
			//case 'SUPPLIER_PROPOSAL_MODIFY':
			//case 'SUPPLIER_PROPOSAL_VALIDATE':
			//case 'SUPPLIER_PROPOSAL_SENTBYMAIL':
			//case 'SUPPLIER_PROPOSAL_CLOSE_SIGNED':
			//case 'SUPPLIER_PROPOSAL_CLOSE_REFUSED':
			//case 'SUPPLIER_PROPOSAL_DELETE':
			//case 'LINESUPPLIER_PROPOSAL_INSERT':
			//case 'LINESUPPLIER_PROPOSAL_UPDATE':
			//case 'LINESUPPLIER_PROPOSAL_DELETE':

			// Contracts
			//case 'CONTRACT_CREATE':
			//case 'CONTRACT_MODIFY':
			//case 'CONTRACT_ACTIVATE':
			//case 'CONTRACT_CANCEL':
			//case 'CONTRACT_CLOSE':
			//case 'CONTRACT_DELETE':
			//case 'LINECONTRACT_INSERT':
			//case 'LINECONTRACT_UPDATE':
			//case 'LINECONTRACT_DELETE':

			// Bills
			//case 'BILL_CREATE':
			//case 'BILL_MODIFY':
			//case 'BILL_VALIDATE':
			//case 'BILL_UNVALIDATE':
			//case 'BILL_SENTBYMAIL':
			//case 'BILL_CANCEL':
			//case 'BILL_DELETE':
			//case 'BILL_PAYED':
			//case 'LINEBILL_INSERT':
			//case 'LINEBILL_UPDATE':
			//case 'LINEBILL_DELETE':

			//Supplier Bill
			//case 'BILL_SUPPLIER_CREATE':
			//case 'BILL_SUPPLIER_UPDATE':
			//case 'BILL_SUPPLIER_DELETE':
			//case 'BILL_SUPPLIER_PAYED':
			//case 'BILL_SUPPLIER_UNPAYED':
			//case 'BILL_SUPPLIER_VALIDATE':
			//case 'BILL_SUPPLIER_UNVALIDATE':
			//case 'LINEBILL_SUPPLIER_CREATE':
			//case 'LINEBILL_SUPPLIER_UPDATE':
			//case 'LINEBILL_SUPPLIER_DELETE':

			// Payments
			//case 'PAYMENT_CUSTOMER_CREATE':
			//case 'PAYMENT_SUPPLIER_CREATE':
			//case 'PAYMENT_ADD_TO_BANK':
			//case 'PAYMENT_DELETE':

			// Online
			//case 'PAYMENT_PAYBOX_OK':
			//case 'PAYMENT_PAYPAL_OK':
			//case 'PAYMENT_STRIPE_OK':

			// Donation
			//case 'DON_CREATE':
			//case 'DON_UPDATE':
			//case 'DON_DELETE':

			// Interventions
			//case 'FICHINTER_CREATE':
			//case 'FICHINTER_MODIFY':
			//case 'FICHINTER_VALIDATE':
			//case 'FICHINTER_DELETE':
			//case 'LINEFICHINTER_CREATE':
			//case 'LINEFICHINTER_UPDATE':
			//case 'LINEFICHINTER_DELETE':

			// Members
			//case 'MEMBER_CREATE':
			//case 'MEMBER_VALIDATE':
			//case 'MEMBER_SUBSCRIPTION':
			//case 'MEMBER_MODIFY':
			//case 'MEMBER_NEW_PASSWORD':
			//case 'MEMBER_RESILIATE':
			//case 'MEMBER_DELETE':

			// Categories
			//case 'CATEGORY_CREATE':
			//case 'CATEGORY_MODIFY':
			//case 'CATEGORY_DELETE':
			//case 'CATEGORY_SET_MULTILANGS':

			// Projects
			//case 'PROJECT_CREATE':
			//case 'PROJECT_MODIFY':
			//case 'PROJECT_DELETE':

			// Project tasks
			//case 'TASK_CREATE':
			//case 'TASK_MODIFY':
			//case 'TASK_DELETE':

			// Task time spent
			//case 'TASK_TIMESPENT_CREATE':
			//case 'TASK_TIMESPENT_MODIFY':
			//case 'TASK_TIMESPENT_DELETE':
			//case 'PROJECT_ADD_CONTACT':
			//case 'PROJECT_DELETE_CONTACT':
			//case 'PROJECT_DELETE_RESOURCE':

			// Shipping
			//case 'SHIPPING_CREATE':
			//case 'SHIPPING_MODIFY':
			case 'SHIPPING_CANCEL':

				$expedition = new Expedition($this->db);
				$expedition->fetch($object->id);


				$expedition->fetch_lines();

				foreach ($expedition->lines as $uneLigneExpedition) {
					$ligneExpedition = new ExpeditionLigne($this->db);
					$ligneExpedition->fetch($uneLigneExpedition->rowid);


					$ligneExpedition->fetchAll();
					$ligneExpedition->fetch_optionals();
					//echo $ligneExpedition->id."<br>";
					if (!empty($ligneExpedition->array_options['options_conditionnement_UC'])) {


						$boxProduct = new Product($this->db);
						$idUC=$ligneExpedition->array_options['options_conditionnement_UC'];

						$nbBox=$ligneExpedition->array_options['options_conditionnement_qtyUC'];

						$boxProduct->fetch($idUC);

						$carton = new Carton($this->db);
						$carton->fetch($idUC);

						$MouvementStock = new MouvementStock($this->db);
						$MouvementStock->origin_id = $expedition->id;
						$MouvementStock->setOrigin("shipping", $expedition->id);


						$idMouvement = $MouvementStock->reception($user, $carton->fk_uc, $ligneExpedition->entrepot_id,$nbBox , 0, $carton->label, dol_now());

					} else {
						setEventMessage("Attention : pour les cartons, une ligne n'a pas été restockée.","warnings");
						continue;
					}


					$expedition->update($user,1);

				}


				break;
			case 'SHIPPING_VALIDATE':

				// Procedure qui met les informations dans les extrafields
				$error=0;
				$expedition = new Expedition($this->db);
				$expedition->fetch($object->id);

				$expedition->fetch_lines();

				$societe = new Societe($this->db);
				$societe->fetch($expedition->socid);
				$societe->fetch_optionals($expedition->socid);


				$idCateg33 = (empty($conf->global->CONDITIONNEMENT_IDCATEG33)) ? 0 : $conf->global->CONDITIONNEMENT_IDCATEG33;

				$idCateg75 = (empty($conf->global->CONDITIONNEMENT_IDCATEG75)) ? 0 : $conf->global->CONDITIONNEMENT_IDCATEG75;



				foreach ($expedition->lines as $currentExpeditionLigne) {
					$expeditionLigne = new ExpeditionLigne($this->db);
					$expeditionLigne->fetch($currentExpeditionLigne->id);


					$sql  = "SELECT categ.rowid as idCateg,categ.label FROM `llx_categorie` as categ";
					$sql .= " INNER JOIN `llx_categorie_product` as categP ON categ.rowid=categP.fk_categorie";
					$sql .= " INNER JOIN `llx_product` as product ON categP.fk_product=product.rowid";
					$sql .= " WHERE product.rowid=".$currentExpeditionLigne->fk_product;
					$sql .= " AND categ.label like '% EN %'";

					$resSQL=$this->db->query($sql);



					foreach ($resSQL as $ligne) {


							if ((int)$ligne['idCateg']==(int)$idCateg33) {
								$idCarton=$societe->array_options['options_conditionnement_UC33'];
							} else {

								if ((int)$ligne['idCateg']==(int)$idCateg75) {
									$idCarton=$societe->array_options['options_conditionnement_UC75'];
								}
							}

						break;
					}




					//$idCarton=$societe->array_options['options_conditionnement_UC'];

					if (empty($idCarton)) {

						if (empty($idCateg75) || empty($idCateg33)) {
							setEventMessage("Attention : Aucun carton n'a été destocker. (Revoir admin du module)", "warnings");
							$error++;
							break;
						} else {
							$error++;

						}


					}

					$carton= new Carton($this->db);
					$carton->fetch($idCarton);

					$boxProduct= new Product($this->db);
					$boxProduct->fetch($carton->fk_product);

					$idUC=$carton->id;

					$MaxInBox = $carton->qtyuc;

					$SubCounter=0;
					$nbBox=0;

					for ($Counter=0;$Counter<=$expeditionLigne->qty;$Counter++) {
						if ($SubCounter==$MaxInBox+1) {
							$nbBox++;
							$SubCounter=1;
						}
						$SubCounter++;
					}

					if ($nbBox*$MaxInBox<$expeditionLigne->qty) {
						$nbBox++;
					}

					//echo $expeditionLigne->id." | $expeditionLigne->qty = $nbBox<br>";
					$expeditionLigne->array_options['options_conditionnement_UC']=$idUC;
					$expeditionLigne->array_options['options_conditionnement_qtyUC']=$nbBox;

					$expeditionLigne->update($user);


					// Procedure qui fait le mouvement de stock de carton

					$idEntrepot=$boxProduct->fk_default_warehouse;

					$MouvementStock = new MouvementStock($this->db);
					$MouvementStock->origin_id=$expedition->id;

					$MouvementStock->setOrigin("shipping",$expedition->id);

					$idMouvement=$MouvementStock->livraison($user,$carton->fk_uc,$idEntrepot,$nbBox,0,$carton->label,dol_now());


				}

				if (empty($error)) {
					setEventMessage("Carton bien déstocké.");
				}




				$expedition->status=1;
				$expedition->update($user,1);

			break;
			//case 'SHIPPING_SENTBYMAIL':
			//case 'SHIPPING_BILLED':
			//case 'SHIPPING_CLOSED':
			//case 'SHIPPING_REOPEN':
			//case 'SHIPPING_DELETE':

			// and more...

			default:
				dol_syslog("Trigger '".$this->name."' for action '$action' launched by ".__FILE__.". id=".$object->id);
				break;
		}

		return 0;
	}
}
